<?php
require_once "../vendor/autoload.php";
require_once "../src/classes/Api.php";
use Aws\S3\S3Client;
use Aws\Exception\AwsException;

// Set up Twig loader system
$loader = new Twig_Loader_Filesystem('../views');
$twig   = new Twig_Environment($loader, array());
$ini_array = parse_ini_file("../conf/phpconfig.ini", true);

//Get data on collections
$api = new Api($ini_array);
$collections = explode(',', $ini_array['collections']);
foreach($collections as $collection){
	$data[$collection] = $api->getContents($collection);
}

/**
*
*	Check if variables that needs to be set is set
*	Return error message and mark fields if not
*
**/
if (empty($_POST['malware_name']) || empty($_FILES['malware_fileToUpload']['tmp_name'])) {
	$data['err_malware_msg'] = "Some fields needs to be filled";
	if(empty($_POST['malware_name'])) {
		$data['err_malware_name'] = "errborder";
	} else {
		$data['malware_name'] = $_POST['malware_name'];
	}
	if (empty($_FILES['malware_fileToUpload']['tmp_name'])) {
		$data['err_malware_file'] = "errborder";
	}
	if (!empty($_POST['malware_summary'])) {
		$data['malware_summary'] = $_POST['malware_summary'];
	}

	$data['malware_platform'] = $_POST['malware_platform'];
	$data['malware_type'] = $_POST['malware_type'];

    echo $twig->render('databaseManagementPage.html', $data); // Render html
}

/**
*
*   If everyhing that needs to be set is set, we can now
*   insert the challenge into swift and mongodb
*
*	Stores:
*	  malware data:
*		- name
*		- summary
*		- platform
*		- type
*
**/
else {
	/** Get Open Stack config **/
    $s3 = include 'openstack/openstack.php';

    /**
    *
    *   File upload to swift container.
    *   Hashed through md5 the filename and a timestamp becomes the id of the mongod document.
    *   to see in swift: swift list TestContainer in cmd
    *
    **/
    $files = [];
    $content = file_get_contents($_FILES['malware_fileToUpload']['tmp_name']);
    $filename = uniqid().'_'.$_FILES['malware_fileToUpload']['name'];
	array_push($files, $filename);

    try{
        $r = $s3->putObject([
            'Bucket' => 'pled_files/malware',
            'Key' => $filename,
            'Body' => $content
        ]);

        /* API post request */
        $json = '{}';
        $resurce = json_decode($json);
        $resurce->resource = [];
        $malware_to_add = json_decode($json);
        $malware_to_add->name = $_POST['malware_name'];
        if (!empty($_POST['malware_summary'])) 
            $malware_to_add->summary = $_POST['malware_summary'];
        if ($_POST['malware_platform'] != "---") 
        	$malware_to_add->platform = $_POST['malware_platform'];
        if ($_POST['malware_type'] != "---") 
        	$malware_to_add->type = $_POST['malware_type'];
        $malware_to_add->file_path = $filename;

        $resource->resource[0] = $malware_to_add;

        //Insert data with api method insert()
        $api = new Api($ini_array);
        $body = json_encode($resource, true);
        $res = $api->insert($body, 'malware');	// call insert function in Api class
        $obj = json_decode($res['response'], true);
        //Check for errors
        if($res['response'] === FALSE){
            die($res['error']);
            $data['error'] = 'dferror';
            echo $twig->render('databaseManagementPage.html', $data); // Render html
        }

        // Error handling for code != 200
        if (!empty($obj['error']['code']) && ($obj['error']['code'] != 200)) {
            $data['uploaded'] = 'false';
        } else {
            $data['uploaded'] = 'true';
        }
        header('Location: ../index.php?uploaded=' . $data['uploaded']);
    } catch (S3Exception $e) {
        echo $e->getMessage() . PHP_EOL;
        $data['error'] = 's3Exeption';
        echo $twig->render('databaseManagementPage.html', $data); // Render html
    }
}